#!/usr/bin/env node
var stringSimilarity = require('string-similarity');
var fs = require("fs");
var userArgs = process.argv.slice(2);

var possible_args = [
    {name: 'create-gulp', call: createGulp},
    {name: 'init-project', call: init},
    {name: 'start', call: startGulp},
    {name: 'save', call: saveGulp},
    {name: '--help', call: help}
];

if(!userArgs[0]){userArgs[0] = "--help"}

var match;
for(var arg = 0; arg < possible_args.length; arg++){
    if(possible_args[arg].name == userArgs[0]){
        match = possible_args[arg];
    }
}

if(!match){
    var simpoints = {points: 0, match: ""};
    for(var arg = 0; arg < possible_args.length; arg++){
        var similarity = stringSimilarity.compareTwoStrings(possible_args[arg].name, userArgs[0]);
        if(similarity > simpoints.points){
            simpoints.points = similarity;
            simpoints.match = possible_args[arg].name;
        }
    }
    console.log("Comando não encontrado, você quis dizer " + simpoints.match + "?");
    console.log("Para consultar os comandos disponíveis utilize --help.")
    process.exit(-1);
}

else{
    match.call();
}

function help(){
    console.log("Possíveis comandos: \n");
    for(var arg = 0; arg < possible_args.length; arg++){
        console.log(possible_args[arg].name);
    }
}

function createGulp(){
    var filename = __dirname + "/assets/default_gulp.txt";
    fs.readFile(filename, 'utf8', function(err, data) {
        if (!err){
            question('Url da página? ', function(answer){
                data = data.replaceAll('{{proxy.url}}', answer);
                fs.writeFile('gulpfile.js', data, function(err) {
                    if(!err){
                        console.log("Gulpfile criado com sucesso.");
                    }
                });
            })
        }
    });
}

function saveGulp(){
    var filename = __dirname + "/assets/projects.json";
    fs.exists(filename, function(exists) {
        if (exists) {
            fs.readFile(filename, 'utf8', function(err, data) {
                data = JSON.parse(data);
                question("Nome do projeto? (usado para iniciar) > ", function(name){
                    question("Path do gulpfile?  (" + process.cwd() + "\\\gulpfile.js) > ", function(path){
                        if(!path){path = process.cwd() + "\\\gulpfile.js";}
                        path = path.replaceAll('\\', "/");
                        data.myprojects.push({name: name, path: path});
                        fs.writeFile(filename, JSON.stringify(data, null, 4), function(err){
                            if(!err){
                                console.log("Projeto " + name + " salvo com sucesso.");
                                process.exit(-1);
                            }
                        });
                    });
                });
            });
        }
        else{
            question("Nome do projeto? (usado para iniciar) > ", function(name){
                question("Path do gulpfile?  (" + process.cwd() + ") > ", function(path){
                    if(!path){path = process.cwd() + "\\\gulpfile.js";}
                    path = path.replaceAll('\\', "/");
                    var myprojects = {myprojects: [{name: name, path: path}]};
                    fs.writeFile(filename, JSON.stringify(myprojects, null, 4), function(err){
                        if(!err){
                            console.log("Projeto " + name + " salvo com sucesso.");
                            process.exit(-1);
                        }
                    });
                });
            });
        }
    });
}

function startGulp(){
    if(!userArgs[1]){
        console.log("Necessário o nome do projeto.");
    }
    else{
        var filename = __dirname + "/assets/projects.json";
        fs.exists(filename, function(exists) {
            if (exists) {
                fs.readFile(filename, 'utf8', function(err, data) {
                    data = JSON.parse(data);
                    var name;
                    for(var i = 0; i < data.myprojects.length; i++){
                        if(data.myprojects[i].name == userArgs[1]){
                            name = data.myprojects[i];
                        }
                    }
                    if(name){
                        var exec = require('child_process').exec;
                        var cmd = 'gulp';
                        exec(cmd, {
                            cwd: name.path
                        }, function(error, stdout, stderr) {
                            console.log(stdout)
                        });
                    }
                    else{
                        console.log("Nenhum projeto " + userArgs[1] + " encontrado.");
                    }
                });
            }
            else{
                console.log("Nenhum projeto encontrado.");
            }
        });
    }
}

function question(question, cb, close) {
    const readline = require('readline');

    const rl = readline.createInterface({
        input: process.stdin,
        output: process.stdout
    });

    rl.question(question, (answer) => {
        cb(answer);
        if(close){
            rl.close();
        }
    });
}

function init(){

}

String.prototype.replaceAll = function(search, replacement) {
    var target = this;
    return target.split(search).join(replacement);
};
